/*
  Copyright NONE this is Free.
  www.byvac.com - see terms and conditions for using hardware
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
  Release Notes:
  March 2018 Firmware version 1.0.0  

*/  
#include "i2cn17.h"

// =============================================================================
// SV3B Type serial control
// =============================================================================
// =============================================================================
// *****************************************************************************
// constructor associates i2c addresses
// *****************************************************************************
I2CN17::I2CN17(int8_t pad)
{
  _PADi2adr=pad;
  Wire.begin(); // join i2c bus
}

// =============================================================================
// Input
// =============================================================================
// =============================================================================

// **************************************************************
// gets a 8 bit value from the i2c bus
// **************************************************************
uint8_t I2CN17::i2_8bit(void)
{
uint8_t rv;
  Wire.requestFrom(_PADi2adr, 1); // returns 1 byte
  rv = Wire.read(); // byte
  Wire.endTransmission();
  return rv;
}

// **************************************************************
// gets a 16 bit value from the i2c bus
// **************************************************************
uint16_t I2CN17::i2_16bit(void)
{
uint16_t rv;
  Wire.requestFrom(_PADi2adr, 2); // returns 2 bytes
  rv=Wire.read()*256; // high byte
  rv+=Wire.read(); // low byte
  Wire.endTransmission();
  return rv;
}

// *****************************************************************************
// send a ommand, single double
// *****************************************************************************
void I2CN17::sendgc(uint8_t cmd)
{
    Wire.beginTransmission(_PADi2adr); 
    Wire.write(cmd); // command
    Wire.endTransmission();
}
void I2CN17::sendgc(uint8_t cmd1,uint8_t cmd2)
{
    Wire.beginTransmission(_PADi2adr); 
    Wire.write(cmd1); // command
    Wire.write(cmd2); // command
    Wire.endTransmission();
}

// ======================== MOTOR ALL ==========================================
// *****************************************************************************
// global enable
// *****************************************************************************
void I2CN17::en(uint8_t on) {
    sendgc(1,on); // command and state
}
// *****************************************************************************
// return status
// *****************************************************************************
uint8_t I2CN17::status(void) {
    sendgc(2);
    return i2_8bit(); 
}
// *****************************************************************************
//  reset all
// *****************************************************************************
void I2CN17::mreset(void) {
    sendgc(3);
}
// *****************************************************************************
//  step all all (6 bytes xH,xL,yH,yL,zH,zL
// *****************************************************************************
void I2CN17::all(uint8_t xH,uint8_t xL,uint8_t yH,uint8_t yL,uint8_t zH,uint8_t zL) {
    Wire.beginTransmission(_PADi2adr); 
    Wire.write(4); // command
    Wire.write(xH);Wire.write(xL);
    Wire.write(yH);Wire.write(yL);
    Wire.write(zH);Wire.write(zL);
    Wire.endTransmission();
}

// ====================== MOTOR X,Y,Z ==========================================
// motor X = 10
//       Y = 20
//       Z = 30
// *****************************************************************************
// set ramp up
// *****************************************************************************
void I2CN17::ru(uint8_t motor, uint8_t value) {
    sendgc(motor,value);
} 
// *****************************************************************************
// set ramp down
// *****************************************************************************
void I2CN17::rd(uint8_t motor, uint8_t value) {
    sendgc(motor+1,value);
} 
// *****************************************************************************
// set speed in sps
// *****************************************************************************
void I2CN17::sps(uint8_t motor, uint16_t value) {
    Wire.beginTransmission(_PADi2adr); 
    Wire.write(motor+2); // command
    Wire.write((uint8_t)(value / 256));    
    Wire.write((uint8_t)(value % 256));    
    Wire.endTransmission();
} 
// *****************************************************************************
// # steps
// *****************************************************************************
void I2CN17::steps(uint8_t motor, uint16_t value) {
    Wire.beginTransmission(_PADi2adr); 
    Wire.write(motor+3); // command
    Wire.write((uint8_t)(value / 256));    
    Wire.write((uint8_t)(value % 256));    
    Wire.endTransmission();
} 
// *****************************************************************************
// direction
// *****************************************************************************
void I2CN17::dir(uint8_t motor, uint8_t value) {
    sendgc(motor+4,value);
} 
// *****************************************************************************
// # steps + direction
// *****************************************************************************
void I2CN17::steps(uint8_t motor, uint16_t steps, uint8_t direction) {
    Wire.beginTransmission(_PADi2adr); 
    Wire.write(motor+5); // command
    Wire.write((uint8_t)(steps / 256)); // steps    
    Wire.write((uint8_t)(steps % 256));    
    Wire.write(direction);
    Wire.endTransmission();
} 
// *****************************************************************************
// return steps to go
// *****************************************************************************
uint16_t I2CN17::togo(uint8_t motor) {
    sendgc(motor+6);
    return i2_16bit(); 
}


// =============================================================================
// S Y S T E M
// =============================================================================
// *****************************************************************************
// restores eeprom contents to factory
// *****************************************************************************
void I2CN17::EEreset(){
    Wire.beginTransmission(0); // general call    
    Wire.write(0x55); // followed by this
    Wire.endTransmission();
}
// *****************************************************************************
// change i2c address
// *****************************************************************************
void I2CN17::change_i2c(uint8_t new_address) {
    sendgc(60,new_address); // cmd, adr        
} 
// *****************************************************************************
// Reads a byte from eeprom at given address
// *****************************************************************************
uint8_t I2CN17::EEread(uint8_t adr)
{
    sendgc(61,adr);
    return i2_8bit();
}
// *****************************************************************************
// Writes a byte to the eeprom at a given address
// *****************************************************************************
void I2CN17::EEwrite(uint8_t adr, uint8_t value)
{
    Wire.beginTransmission(_PADi2adr);
    Wire.write(62); // command
    Wire.write(adr);
    Wire.write(value);
    Wire.endTransmission();
}
// *****************************************************************************
// Get device ID as a 16 bit number
// *****************************************************************************
uint16_t I2CN17::ID()
{
    sendgc(63);
    return i2_16bit();
}
// *****************************************************************************
// Get firmware version returns 3 bytes
// *****************************************************************************
void I2CN17::Version(uint8_t *b)
{
    sendgc(64);
    Wire.requestFrom(_PADi2adr, 2); // returns 16 bytes
    *(b++)=Wire.read();
    *(b++)=Wire.read();   
    *(b)=Wire.read();   
}
// *****************************************************************************
// reset
// *****************************************************************************
void I2CN17::reset(void) {
    sendgc(66);         
} 










