/*
  User Interface with touch pad and lcd display
  BV4244 
  Copyright (c) 2013 Jim Spence.  All right reserved.
  www.byvac.com - see terms and conditions for using hardware
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/  

// See cpp for release notes

#ifndef _H_i2cn17_h
#define _H_i2cn17_h


#include "Arduino.h"
#include "Wire.h"

// macro *** Bit # from 0 to 7
#define bitset(var,bitno) ((var) |= 1UL << (bitno))
#define bitclr(var,bitno) ((var) &= ~(1UL << (bitno)))

class I2CN17
{
    public:
        // non-device specific
        I2CN17(int8_t pad);
        // all
        void en(uint8_t on);
        uint8_t status(void);
        void mreset(void);
        void all(uint8_t xH,uint8_t xL,uint8_t yH,uint8_t yL,uint8_t zH,uint8_t zL);
        // motor
        void ru(uint8_t motor, uint8_t value);
        void rd(uint8_t motor, uint8_t value);
        void sps(uint8_t motor, uint16_t value);
        void steps(uint8_t motor, uint16_t value);
        void dir(uint8_t motor, uint8_t value);
        void steps(uint8_t motor, uint16_t steps, uint8_t direction);
        uint16_t togo(uint8_t motor);
        // system section
        void EEreset();
        void change_i2c(uint8_t new_address);
        void EEwrite(uint8_t adr, uint8_t value);
        uint8_t EEread(uint8_t adr);
        uint16_t ID();
        void Version(uint8_t *b);
        void reset(void);
    private:
        int8_t _PADi2adr;
        void sendgc(uint8_t cmd);
        void sendgc(uint8_t cmd1, uint8_t cmd2);
        uint8_t i2_8bit();
        uint16_t i2_16bit();
};


#endif